@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python3
 
 import sys
 import gettext
@@ -7,11 +7,14 @@ gi.require_version('Gtk', '3.0')
 gi.require_version('GnomeBluetooth', '1.0')
 from gi.repository import Gtk, Gdk, GnomeBluetooth, Gio
 import rfkillMagic
+import setproctitle
 import subprocess
 
 # i18n
 gettext.install("blueberry", "/usr/share/locale")
 
+setproctitle.setproctitle("blueberry-tray")
+
 class BluetoothTray(Gtk.Application):
     def __init__(self):
         super(BluetoothTray, self).__init__(register_session=True, application_id="org.linuxmint.blueberry.tray")
@@ -27,7 +30,7 @@ class BluetoothTray(Gtk.Application):
             debug = True
 
         self.rfkill = rfkillMagic.Interface(self.update_icon_callback, debug)
-        self.settings = Gio.Settings("org.blueberry")
+        self.settings = Gio.Settings(schema="org.blueberry")
         self.settings.connect("changed::tray-enabled", self.on_settings_changed_cb)
 
         # If we have no adapter, or disabled tray, end early
@@ -65,17 +68,18 @@ class BluetoothTray(Gtk.Application):
             self.update_connected_state()
 
     def update_connected_state(self):
-        connected_devices = self.get_connected_devices()
+        self.get_devices()
 
-        if len(connected_devices) > 0:
+        if len(self.connected_devices) > 0:
             self.icon.set_from_icon_name("blueberry-tray-active")
-            self.icon.set_tooltip_text(_("Bluetooth: Connected to %s") % (", ".join(connected_devices)))
+            self.icon.set_tooltip_text(_("Bluetooth: Connected to %s") % (", ".join(self.connected_devices)))
         else:
             self.icon.set_from_icon_name("blueberry-tray")
             self.icon.set_tooltip_text(_("Bluetooth"))
 
-    def get_connected_devices(self):
-        connected_devices = []
+    def get_devices(self):
+        self.connected_devices = []
+        self.paired_devices = {}
         default_iter = None
 
         iter = self.model.get_iter_first()
@@ -93,12 +97,16 @@ class BluetoothTray(Gtk.Application):
                 connected = self.model.get_value(iter, GnomeBluetooth.Column.CONNECTED)
                 if connected:
                     name = self.model.get_value(iter, GnomeBluetooth.Column.NAME)
-                    connected_devices.append(name)
+                    self.connected_devices.append(name)
+
+                paired = self.model.get_value(iter, GnomeBluetooth.Column.PAIRED)
+                if paired:
+                    name = self.model.get_value(iter, GnomeBluetooth.Column.NAME)
+                    proxy = self.model.get_value(iter, GnomeBluetooth.Column.PROXY)
+                    self.paired_devices[name] = proxy
 
                 iter = self.model.iter_next(iter)
 
-        return connected_devices
-
     def on_activate(self, icon, data=None):
         subprocess.Popen(["blueberry"])
 
@@ -120,6 +128,23 @@ class BluetoothTray(Gtk.Application):
         item.connect("activate", self.open_manager_cb)
         menu.append(item)
 
+        if len(self.paired_devices) > 0:
+            menu.append(Gtk.SeparatorMenuItem())
+            m_item = Gtk.MenuItem(_("Paired devices"))
+            menu.append(m_item)
+            paired_menu = Gtk.Menu()
+            m_item.set_submenu(paired_menu)
+            for device in self.paired_devices:
+                label = device
+                item = Gtk.ImageMenuItem(label=label)
+                if device in self.connected_devices:
+                    image = Gtk.Image.new_from_icon_name("emblem-ok-symbolic", Gtk.IconSize.MENU)
+                    image.set_tooltip_text(_("Connected"))
+                    item.set_always_show_image(True)
+                    item.set_image(image)
+                item.connect("activate",self.toggle_connect_cb, device)
+                paired_menu.append(item)
+
         menu.append(Gtk.SeparatorMenuItem())
 
         item = Gtk.MenuItem(label=_("Quit"))
@@ -131,6 +156,11 @@ class BluetoothTray(Gtk.Application):
         device = Gdk.Display.get_default().get_device_manager().get_client_pointer()
         menu.popup_for_device(device, None, None, position_menu_cb, icon, button, time)
 
+    def toggle_connect_cb(self, item, data = None):
+        proxy = self.paired_devices[data]
+        connected = data in self.connected_devices
+        self.client.connect_service(proxy.get_object_path(), not connected)
+
     def send_files_cb(self, item, data = None):
         subprocess.Popen(["bluetooth-sendto"])
 
